@@ -1,7 +1,7 @@
 /* Copyright (C) 1997, 1998, 1999 Marius Vollmer
  * Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
  *
- * $Id: rep-gtk.c,v 1.32 2000/04/09 17:32:28 john Exp $
+ * $Id: rep-gtk.c,v 1.39 2000/09/18 13:05:15 john Exp $
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -26,6 +26,10 @@
 #include <string.h>
 #include <limits.h>
 
+#ifdef HAVE_LOCALE_H
+# include <locale.h>
+#endif
+
 /* Define this to enable some output during GC and other interesting
    actions. */
 #undef DEBUG_PRINT
@@ -47,24 +51,11 @@ list_length (repv list)
 
 static GHashTable *proxy_tab;
 
-static guint
-gpointer_hash (gpointer a)
-{
-  return (guint)a;
-}
-
-static gint
-gpointer_compare (gpointer a, gpointer b)
-{
-  return a == b;
-}
-
 static void
 enter_proxy (gpointer obj, repv proxy)
 {
   if (proxy_tab == NULL)
-    proxy_tab = g_hash_table_new ((GHashFunc)gpointer_hash,
-				  (GCompareFunc)gpointer_compare);
+    proxy_tab = g_hash_table_new (NULL, NULL);
   g_hash_table_insert (proxy_tab, obj, (gpointer)proxy);
 }
 
@@ -196,6 +187,7 @@ sgtk_try_missing_type (char *name)
 {
   static sgtk_type_info missing[] = {
     { "GdkGC", GTK_TYPE_BOXED },
+    { "GdkPixbuf", GTK_TYPE_BOXED },	/* XXX okay? */
     { "GtkToolbarStyle", GTK_TYPE_ENUM },
     { "GtkToolbarChildType", GTK_TYPE_ENUM },
     { "GtkTreeViewMode", GTK_TYPE_ENUM },
@@ -234,6 +226,7 @@ sgtk_fillin_type_info (sgtk_type_info *info)
 	this_type = sgtk_try_missing_type (info->name);
       if (this_type == GTK_TYPE_INVALID)
 	{
+	  if (info->type == GTK_TYPE_BOXED)
 	  fprintf (stderr, "unknown type `%s'.\n", info->name);
 	  return 0;
 	}
@@ -497,12 +490,17 @@ gtkobj_print (repv stream, repv obj)
     char buf[32];
   sgtk_object_proxy *proxy = GTKOBJ_PROXY (obj);
   GtkType tid = GTK_OBJECT_TYPE (proxy->obj);
-
+  char *type = gtk_type_name (tid);
   rep_stream_puts (stream, "#<", -1, rep_FALSE);
-  rep_stream_puts (stream, gtk_type_name (tid), -1, rep_FALSE);
+  rep_stream_puts (stream, type ? type : "Gtk???", -1, rep_FALSE);
   rep_stream_puts (stream, " ", -1, rep_FALSE);
-  sprintf (buf, "%lx", (long)proxy->obj);
-  rep_stream_puts (stream, buf, -1, rep_FALSE);
+  if (GTK_OBJECT_DESTROYED (proxy->obj))
+      rep_stream_puts (stream, "destroyed", -1, rep_FALSE);
+  else
+  {
+      sprintf (buf, "%lx", (long)proxy->obj);
+      rep_stream_puts (stream, buf, -1, rep_FALSE);
+  }
   rep_stream_putc (stream, '>');
 }
 
@@ -624,6 +622,8 @@ make_gtkobj (GtkObject *obj)
 {
   sgtk_object_proxy *proxy;
 
+  g_assert (obj->ref_count > 0);
+
   proxy = (sgtk_object_proxy *)rep_ALLOC_CELL (sizeof(sgtk_object_proxy));
   gtk_object_ref (obj);
   gtk_object_sink (obj);
@@ -663,8 +663,13 @@ sgtk_wrap_gtkobj (GtkObject *obj)
 int
 sgtk_is_a_gtkobj (guint type, repv obj)
 {
-  if (!(GTKOBJP (obj)))
-    return 0;
+  if (!GTKOBJP (obj)
+      || !GTK_IS_OBJECT (GTKOBJ_PROXY(obj)->obj)
+      || GTK_OBJECT_DESTROYED (GTKOBJ_PROXY(obj)->obj))
+    {
+      return 0;
+    }
+
   return gtk_type_is_a (GTK_OBJECT_TYPE(GTKOBJ_PROXY(obj)->obj), type);
 }
 
@@ -842,14 +847,14 @@ char *
 sgtk_rep_to_senum (repv obj, sgtk_senum_info *info)
 {
   int i;
+  char *obj_name;
 
   if (rep_STRINGP (obj))
-    {
-      return rep_STR (obj);
-    }
+    return rep_STR (obj);
 
+  obj_name = rep_STR (rep_SYM (obj)->name);
   for (i = 0; i < info->n_literals; i++)
-    if (! strcmp (info->literals[i].name, rep_STR (obj)))
+    if (! strcmp (info->literals[i].name, obj_name))
       return info->literals[i].value;
   return NULL;
 }
@@ -1121,6 +1126,8 @@ sgtk_arg_to_rep (GtkArg *a, int free_mem)
       return sgtk_boxed_to_rep (GTK_VALUE_BOXED(*a),
 				(sgtk_boxed_info *)sgtk_find_type_info (a->type),
 				TRUE);
+    case GTK_TYPE_POINTER:
+      return sgtk_pointer_to_rep (GTK_VALUE_POINTER(*a));
     case GTK_TYPE_OBJECT:
       return sgtk_wrap_gtkobj (GTK_VALUE_OBJECT(*a));
     default:
@@ -1161,6 +1168,9 @@ sgtk_valid_arg (GtkArg *a, repv obj)
       return sgtk_valid_boxed (obj, ((sgtk_boxed_info *)
 				     sgtk_find_type_info (a->type)));
       break;
+    case GTK_TYPE_POINTER:
+      return BOXED_P (obj) || GTKOBJP (obj) || sgtk_valid_pointer (obj);
+      break;
     case GTK_TYPE_CALLBACK:
       return sgtk_valid_function (obj);
     case GTK_TYPE_OBJECT:
@@ -1216,6 +1226,14 @@ sgtk_rep_to_arg (GtkArg *a, repv obj, repv protector)
     case GTK_TYPE_BOXED:
       GTK_VALUE_BOXED(*a) = sgtk_rep_to_boxed (obj);
       break;
+    case GTK_TYPE_POINTER:
+      if (BOXED_P (obj))
+	  GTK_VALUE_POINTER(*a) = BOXED_PTR (obj);
+      else if (GTKOBJP (obj))
+	  GTK_VALUE_POINTER(*a) = GTKOBJ_PROXY (obj)->obj;
+      else
+	  GTK_VALUE_POINTER(*a) = sgtk_rep_to_pointer (obj);
+      break;
     case GTK_TYPE_CALLBACK:
       sgtk_protect (protector, obj);
       GTK_VALUE_CALLBACK(*a).marshal = sgtk_callback_marshal;
@@ -1878,7 +1896,7 @@ static void
 sgtk_input_callback (gpointer data, gint fd, GdkInputCondition cond)
 {
     struct input_callback_data d;
-    d.func = g_hash_table_lookup (input_callbacks, (gpointer) fd);
+    d.func = g_hash_table_lookup (input_callbacks, GINT_TO_POINTER (fd));
     d.fd = fd;
     if (d.func != 0)
     {
@@ -1896,16 +1914,14 @@ sgtk_register_input_fd (int fd, void (*callback)(int fd))
 	int tag;
 	if (input_tags == 0)
 	{
-	    input_tags = g_hash_table_new ((GHashFunc) gpointer_hash,
-					   (GCompareFunc) gpointer_compare);
-	    input_callbacks = g_hash_table_new ((GHashFunc) gpointer_hash,
-						(GCompareFunc) gpointer_compare);
+	    input_tags = g_hash_table_new (NULL, NULL);
+	    input_callbacks = g_hash_table_new (NULL, NULL);
 	}
 	tag = gdk_input_add (fd, GDK_INPUT_READ,
 			     (GdkInputFunction) sgtk_input_callback, 0);
-	g_hash_table_insert (input_tags, (gpointer) fd, (gpointer) tag);
+	g_hash_table_insert (input_tags, GINT_TO_POINTER (fd), GINT_TO_POINTER (tag));
 	g_hash_table_insert (input_callbacks,
-			     (gpointer) fd, (gpointer) callback);
+			     GINT_TO_POINTER (fd), (gpointer) callback);
     }
 }
 
@@ -1914,10 +1930,10 @@ sgtk_deregister_input_fd (int fd)
 {
     if (input_tags != 0)
     {
-	int tag = (int) g_hash_table_lookup (input_tags, (gpointer) fd);
+	int tag = GPOINTER_TO_INT (g_hash_table_lookup (input_tags, GINT_TO_POINTER (fd)));
 	gdk_input_remove (tag);
-	g_hash_table_remove (input_tags, (gpointer) fd);
-	g_hash_table_remove (input_callbacks, (gpointer) fd);
+	g_hash_table_remove (input_tags, GINT_TO_POINTER (fd));
+	g_hash_table_remove (input_callbacks, GINT_TO_POINTER (fd));
     }
 }
 
@@ -2082,11 +2098,13 @@ DEFUN ("gtk-standalone-p", Fgtk_standalone_p,
 DEFSYM (gtk_major_version, "gtk-major-version");
 DEFSYM (gtk_minor_version, "gtk-minor-version");
 DEFSYM (gtk_micro_version, "gtk-micro-version");
-DEFSYM (gtk, "gtk");
+DEFSYM (rep_gtk_version, "rep-gtk-version");
 
 static void
 sgtk_init_substrate (void)
 {
+  DEFSTRING (ver, REP_GTK_VERSION);
+
   tc16_gtkobj = rep_register_new_type ("gtk-object", 0,
 				       gtkobj_print, gtkobj_print,
 				       gtkobj_sweep, gtkobj_mark,
@@ -2123,10 +2141,15 @@ sgtk_init_substrate (void)
   rep_INTERN (gtk_major_version);
   rep_INTERN (gtk_minor_version);
   rep_INTERN (gtk_micro_version);
-  rep_INTERN (gtk);
+  rep_INTERN (rep_gtk_version);
   Fset (Qgtk_major_version, rep_MAKE_INT (GTK_MAJOR_VERSION));
   Fset (Qgtk_minor_version, rep_MAKE_INT (GTK_MINOR_VERSION));
   Fset (Qgtk_micro_version, rep_MAKE_INT (GTK_MICRO_VERSION));
+  Fset (Qrep_gtk_version, rep_VAL (&ver));
+  Fexport_bindings (rep_list_4 (Qgtk_major_version,
+				Qgtk_minor_version,
+				Qgtk_micro_version,
+				Qrep_gtk_version));
 }
 
 static int sgtk_inited = 0;
@@ -2151,6 +2174,11 @@ sgtk_init_with_args (int *argcp, char ***argvp)
         {
 	  gtk_set_locale ();
 	  gtk_init (argcp, argvp);
+
+#ifdef HAVE_SETLOCALE
+	  /* XXX remove when no longer needed.. */
+	  setlocale (LC_NUMERIC, "C");
+#endif
         }
     }
 
@@ -2240,17 +2268,13 @@ extern void sgtk_init_gtk_gtk_glue (void);
 repv
 rep_dl_init (void)
 {
-#if rep_INTERFACE >= 9
-  repv tem = rep_push_structure ("gtk");
-#endif
-
+  repv tem = rep_push_structure ("gui.gtk.gtk");
+  /* ::alias:gui.gtk gui.gtk.gtk::
+     ::alias:gtk gui.gtk.gtk:: */
+  rep_alias_structure ("gui.gtk");
+  rep_alias_structure ("gtk");
   sgtk_init_gtk_gtk_glue ();
-
-#if rep_INTERFACE >= 9
   return rep_pop_structure (tem);
-#else
-  return Qgtk;
-#endif
 }
 
 /* This is required mainly since other dls may try to unregister
