@@ -1,7 +1,7 @@
 /* Copyright (C) 1997, 1998, 1999 Marius Vollmer
  * Copyright (C) 1999 John Harper <john@dcs.warwick.ac.uk>
  *
- * $Id: rep-gtk.h,v 1.14 2000/03/06 11:43:12 john Exp $
+ * $Id: rep-gtk.h,v 1.16 2000/07/09 11:05:02 john Exp $
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,10 +28,6 @@
 #include <rep.h>
 #include <gtk/gtk.h>
 
-#ifndef rep_INTERFACE
-# define rep_INTERFACE 7
-#endif
-
 typedef struct _sgtk_type_info {
   char *name;
   GtkType type;
@@ -100,6 +96,7 @@ gint sgtk_rep_to_int (repv obj);
 gulong sgtk_rep_to_ulong (repv obj);
 glong sgtk_rep_to_long (repv obj);
 gchar sgtk_rep_to_char (repv obj);
+repv sgtk_char_to_rep (gchar c);
 char *sgtk_rep_to_string (repv obj);
 repv sgtk_string_to_rep (char *x);
 repv sgtk_static_string_to_rep (const char *x);
